﻿using Dss.Api.Examples.Properties;
using System;
using System.Net;
using DataScope.Select.Api.EvaluationConnect;
using DataScope.Select.Api.Authentication;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Jobs;
using DataScope.Select.Api.Quota;
using DataScope.Select.Api.Search;
using DataScope.Select.Api.StandardExtractions;
using DataScope.Select.Api.Users;

namespace Dss.Api.Examples
{
    static public class ContextHelper
    {

        private static string _token;

        public static void ClearToken()
        {
            _token = null;
        }

        public static JobsContext CreateJobsContext()
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            return new JobsContext(serverUri, _token, () => creds, token => _token = token );
        }

        public static EvaluationConnectContext CreateEvaluationConnectContext()
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            //NOTE: Sample Mode is set to true for the EvaluationConnectContext because without this the example application would be
            //submitting actual challenges for which evaluators will be responding.
            return new EvaluationConnectContext(serverUri, _token, () => creds, token => _token = token, true);
        }

        public static ExtractionsContext CreateExtractionsContext()
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            return new ExtractionsContext(serverUri, _token, () => creds, token => _token = token);
        }

        public static StandardExtractionsContext CreateStandardExtractionsContext()
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            return new StandardExtractionsContext(serverUri, _token, () => creds, token => _token = token);
        }

        public static SearchContext CreateSearchContext()
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            return new SearchContext(serverUri, _token, () => creds, token => _token = token);
        }

        public static QuotaContext CreateQuotaContext()
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            return new QuotaContext(serverUri, _token, () => creds, token => _token = token, true);
        }

        public static UsersContext CreateUsersContext(bool authenticate = true)
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            var context = new UsersContext(serverUri, _token, () => creds, token => _token = token);

            if (authenticate)
                _token = context.SessionToken;

            return context;
        }

        public static AuthenticationContext CreateAuthenticationContext(bool authenticate = true)
        {
            var serverUri = new Uri(Settings.Default.ServiceUri);
            var creds = new NetworkCredential(Settings.Default.UserId, Settings.Default.Password);

            var context = new AuthenticationContext(serverUri, _token, () => creds, token => _token = token);

            if (authenticate)
                _token = context.SessionToken;

            return context;
        }

    }
}
