﻿using Dss.Api.Examples.Internal;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.SubjectLists;

namespace Dss.Api.Examples.Extractions.SubjectLists.InstrumentLists
{

    [Example("Instrument List Examples")]
    public class InstrumentListExamples
    {

        #region Get

        [Example("Get All (Paged Results)")]
        public void GetAllPagedResults()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            //Set the page size to 1 record per page for the purposes of this example
            extractionsContext.Preferences.MaxPageSize = 1;

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create an instrument list 
            extractionsContext.InstrumentListOperations.Create(new InstrumentList { Name = "GetAllPagedResults" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create another instrument list 
            extractionsContext.InstrumentListOperations.Create(new InstrumentList { Name = "GetAllPagedResults" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAll", MethodType.Operation, Publish.Primary);
            //Get the first page of instrument lists (default page size is 250)
            var page1Results = extractionsContext.InstrumentListOperations.GetAll();

            //Output
            Status.WriteLine("HasNextPage: {0}", page1Results.HasNextPage);
            Status.WriteLine("NextLink.Url: {0}", page1Results.NextLink.Url);
            Status.WriteLine("Count: {0}", page1Results.Count());
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAll", MethodType.Operation, Publish.Primary);
            if (page1Results.HasNextPage) //Use a while loop if you need to retrieve all instrument lists
            {
                //Get the second page of instrument lists by passing in the NextLink returned from the previous request
                var page2Results = extractionsContext.InstrumentListOperations.GetAll(page1Results.NextLink);

                //Output
                Status.WriteLine("HasNextPage: {0}", page2Results.HasNextPage);
                Status.WriteLine("NextLink.Url: {0}", page2Results.NextLink.Url);
                Status.WriteLine("Count: {0}", page2Results.Count());
            }
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to retrieve an existing InstrumentList by its name.
        /// </summary>
        [Example("Get By Name")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(InstrumentList))]
        public void GetByName()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var name = "GetByName" + Suffix.New();
            var instrumentList = new InstrumentList { Name = name };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetByName", MethodType.Operation, Publish.Primary);
            var createdInstrumentList = extractionsContext.InstrumentListOperations.GetByName(name);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region CUD

        /// <summary>
        /// This example demonstrates how to create an InstrumentList.
        /// </summary>
        [Example("Create")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(InstrumentList))]
        public void Create()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create", MethodType.Operation, Publish.Primary);
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to update an InstrumentList.
        /// </summary>
        [Example("Update")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(InstrumentList))]
        public void Update()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Update", MethodType.Operation, Publish.Primary);
            createdInstrumentList.Name = "Updated Instrument List Name";
            extractionsContext.InstrumentListOperations.Update(createdInstrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to delete an InstrumentList.
        /// </summary>
        [Example("Delete")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(InstrumentList))]
        public void Delete()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete", MethodType.Operation, Publish.Primary);
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Items

        /// <summary>
        /// This example demonstrates how to append identifiers to an existing InstrumentList.
        /// </summary>
        [Example("Append Identifiers")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(InstrumentList))]
        public void AppendIdentifiers()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers", MethodType.Operation, Publish.Primary);
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[]
            {
                new InstrumentIdentifier
                {
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
                },
                // This next entry will cause a warning message in the AppendIdentifiers results
                // and this UDI will be dropped in the GetAllEntities results
                new InstrumentIdentifier
                {
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "ßÃÐ"
                }
            };
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, true);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAllEntities");
            extractionsContext.InstrumentListOperations.GetAllInstruments(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to update a specific entity in an existing InstrumentList.
        /// </summary>
        [Example("Update Item")]
        public void UpdateItem()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {new InstrumentIdentifier
                {
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
                }};
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAllInstruments");
            var instruments = extractionsContext.InstrumentListOperations.GetAllInstruments(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "UpdateItem", MethodType.Operation, Publish.Primary);
            var instrumentListItem = instruments.First(i => i.Identifier == "IBM.N");
            instrumentListItem.UserDefinedIdentifier = "1";
            extractionsContext.InstrumentListOperations.UpdateItem(instrumentListItem);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to delete a specific entity in an existing InstrumentList.
        /// </summary>
        [Example("Delete Item")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(InstrumentList))]
        public void DeleteItem()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {new InstrumentIdentifier
                {
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
                }};
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAllInstruments");
            var instruments = extractionsContext.InstrumentListOperations.GetAllInstruments(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "DeleteItem", MethodType.Operation, Publish.Primary);
            var instrumentListItem = instruments.First(i => i.Identifier == "IBM.N");
            extractionsContext.InstrumentListOperations.DeleteItem(instrumentListItem);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to delete a specific entity in an existing InstrumentList.
        /// </summary>
        [Example("Append Identifiers: Validation Errors")]
        public void ValidationErrors()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Instrument List Name" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers", MethodType.Operation, Publish.Secondary);
            //Append a bad identifier which will return a validation error
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {
                new InstrumentIdentifier
                {
                    Identifier = "@@@",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
                }};
            var result = extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);

            //Output
            foreach (var message in result.ValidationResult.Messages)
                Status.WriteLine("{0} ({1})", message.Message, message.Severity);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            var createdInstrumentList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(createdInstrumentList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Export / Import

        [Example("Export: Xml")]
        public void ExportXml()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "ExportXml" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {new InstrumentIdentifier
				{
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Export", MethodType.Operation, Publish.Primary);
            var exportBytes = extractionsContext.InstrumentListOperations.Export(
                instrumentList.ListId, InstrumentListExportFormat.Xml, true /*Include source */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(instrumentList);
            Status.EndNotify(extractionsContext);

        }

        [Example("Import: Xml")]
        public void ImportXml()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "ImportXml" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {new InstrumentIdentifier
				{
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Export", MethodType.Operation, Publish.Secondary);
            var exportBytes = extractionsContext.InstrumentListOperations.Export(
                instrumentList.ListId, InstrumentListExportFormat.Xml, true /* include source */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Export", MethodType.Operation, Publish.Primary);
            var importBytes = Encoding.UTF8.GetBytes(exportString);
            var listName = "ImportXmlNew" + Suffix.New() + ".xml";
            extractionsContext.InstrumentListOperations.ImportFromBytes(
                listName,
                importBytes,
                new InstrumentListImportSettings
                {
                    AllowDuplicateInstruments = false,
                    ImportUserInstruments = false,
                    PredefinedAction = ImportActionEnum.Append,
                    PredefinedName = listName
                });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetByName");
            //When a list is imported, we don't know what the id of the report template is and therefore
            //we need to retrieve the report template by name.
            var newList = extractionsContext.InstrumentListOperations.GetByName(listName);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(newList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Export: Csv")]
        public void ExportCsv()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "Export" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {new InstrumentIdentifier
				{
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Export", MethodType.Operation, Publish.Primary);
            var exportBytes = extractionsContext.InstrumentListOperations.Export(
                instrumentList.ListId, InstrumentListExportFormat.Csv, true /* include source */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(instrumentList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Import: Csv")]
        public void ImportCsv()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            var instrumentList = new InstrumentList { Name = "ImportXml" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            IEnumerable<InstrumentIdentifier> appendIdentifiers = new[] {new InstrumentIdentifier
				{
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, appendIdentifiers, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Export", MethodType.Operation, Publish.Secondary);
            var exportBytes = extractionsContext.InstrumentListOperations.Export(
                instrumentList.ListId, InstrumentListExportFormat.Csv, true /* include source */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Export", MethodType.Operation, Publish.Primary);
            var importBytes = Encoding.UTF8.GetBytes(exportString);
            var listName = "ImportXmlNew" + Suffix.New() + ".csv";
            extractionsContext.InstrumentListOperations.ImportFromBytes(
                listName,
                importBytes,
                new InstrumentListImportSettings
                {
                    AllowDuplicateInstruments = false,
                    ImportUserInstruments = false,
                    PredefinedAction = ImportActionEnum.Append,
                    PredefinedName = listName
                });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetByName");
            //When a list is imported, we don't know what the id of the report template is and therefore
            //we need to retrieve the report template by name.
            var newList = extractionsContext.InstrumentListOperations.GetByName(listName);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Delete");
            extractionsContext.InstrumentListOperations.Delete(newList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

    }

}
