﻿using ICSharpCode.AvalonEdit.Rendering;
using System.Linq;
using System.Windows;
using System.Windows.Media;

namespace Dss.Api.Examples.Internal
{

    public class DiffLineBackgroundRenderer : IBackgroundRenderer
    {
        private readonly int? _highlightFrom;
        private readonly int? _highlightTo;

        private static readonly Pen _pen;
        private static readonly SolidColorBrush _highlightBrush;

        static DiffLineBackgroundRenderer()
        {
            _highlightBrush = new SolidColorBrush(new Color { R = 0xff, G = 0xff, B=0xa1, A = 0xff});
            _highlightBrush.Freeze();
            
            var blackBrush = new SolidColorBrush(Color.FromRgb(0, 0, 0));
            blackBrush.Freeze();
            _pen = new Pen(blackBrush, 0.0);
        }

        public DiffLineBackgroundRenderer(int? highlightFrom, int? highlightTo)
        {
            _highlightFrom = highlightFrom;
            _highlightTo = highlightTo;
        }

        public KnownLayer Layer
        {
            get { return KnownLayer.Background; }
        }

        public void Draw(TextView textView, DrawingContext drawingContext)
        {
            if (_highlightFrom == null || _highlightTo == null)
                return;
            foreach (var visualLine in textView.VisualLines)
            {
                var lineNo = visualLine.FirstDocumentLine.LineNumber-1; //Snippet lines are zero-based
                if (lineNo >= _highlightFrom.Value && lineNo <= _highlightTo.Value)
                {
                    var r = BackgroundGeometryBuilder.GetRectsFromVisualSegment(textView, visualLine, 1, 1).FirstOrDefault();
                    drawingContext.DrawRectangle(_highlightBrush, _pen, new Rect(0, r.Top-1, textView.ActualWidth, r.Height+1));
                }
            }
        }

    }

}
