﻿using System.Text;
using Dss.Api.Examples.Internal;
using System;
using System.Linq;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Content.Equity;
using DataScope.Select.Api.Content.FixedIncome.GovCorp;
using DataScope.Select.Api.Content.Fund;
using DataScope.Select.Api.Search;
using DataScope.Select.Core.RestApi;

namespace Dss.Api.Examples.Search
{
    
    [Example("Search")]
    public class Search
    {

        protected SearchContext SearchContext { get; set; }

        public Search()
        {
            SearchContext = ContextHelper.CreateSearchContext();
        }

        #region All Search

        [Example("All Search: By RIC")]
        public void AllSearch_ByRic()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "InstrumentSearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.InstrumentSearch(
                AllInstrumentSearchRequest.Create(
                    new[] { 
                        InstrumentTypeGroup.CollatetizedMortgageObligations, InstrumentTypeGroup.Commodities, InstrumentTypeGroup.Equities, 
                        InstrumentTypeGroup.FuturesAndOptions, InstrumentTypeGroup.GovCorp, InstrumentTypeGroup.MortgageBackedSecurities,
                        InstrumentTypeGroup.Money, InstrumentTypeGroup.Municipals, InstrumentTypeGroup.Funds },
                    IdentifierType.Ric, 
                    "US10YT=RRPS",
                    IdentifierType.Ric)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        [Example("All Search: By ORG ID")]
        public void AllSearch_ByOrgId()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "InstrumentSearch", MethodType.Operation, Publish.Secondary);
            var results = SearchContext.InstrumentSearch(
                AllInstrumentSearchRequest.Create(
                    new[]
                    {
                        InstrumentTypeGroup.CollatetizedMortgageObligations, InstrumentTypeGroup.Commodities, InstrumentTypeGroup.Equities,
                        InstrumentTypeGroup.FuturesAndOptions, InstrumentTypeGroup.GovCorp,
                        InstrumentTypeGroup.MortgageBackedSecurities,
                        InstrumentTypeGroup.Money, InstrumentTypeGroup.Municipals, InstrumentTypeGroup.Funds
                    },
                    IdentifierType.OrgId,
                    "97312,18175",
                    IdentifierType.OrgId)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        [Example("All Search: Wild Card Suffix")]
        public void AllSearch_WildCardSuffix()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "InstrumentSearch", MethodType.Operation, Publish.Secondary);
            var results = SearchContext.InstrumentSearch(
                AllInstrumentSearchRequest.Create(
                    new[]
                    {
                        InstrumentTypeGroup.CollatetizedMortgageObligations, InstrumentTypeGroup.Commodities, InstrumentTypeGroup.Equities,
                        InstrumentTypeGroup.FuturesAndOptions, InstrumentTypeGroup.GovCorp,
                        InstrumentTypeGroup.MortgageBackedSecurities,
                        InstrumentTypeGroup.Money, InstrumentTypeGroup.Municipals, InstrumentTypeGroup.Funds
                    },
                    IdentifierType.OrgId,
                    "9731*",
                    IdentifierType.ISMA)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

		#region Classifications

		[Example("Business Classification: Get Refinitiv Business Classifications")]
		public void GetTrBusinessClassifications()
		{
			SearchContext.Preferences.MaxPageSize = 10;

			Status.Notify(SearchContext, null, "GetTrBusinessClassifications", MethodType.Operation, Publish.Primary);
			var contributors = SearchContext.GetTrBusinessClassifications();
			Status.WriteLine("GetTrBusinessClassifications: " +
							String.Join(", ", contributors.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);
		}

		#endregion

        #region Entity Search

        [Example("Entity Search: By RIC")]
        public void EntitySearch_ByRic()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "EntitySearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.EntitySearch(
                EntitySearchRequest.Create(
                    null, //Name
                    false, //Offical name only
                    null, //Domicile codes
                    null, //Country of incorporation
                    null, //Industry classification
                    false, //Verified only
                    false, //Regulated only
                    false, //Issuers only
                    false, //Active only
                    false, //includeUnManagedOrUnVerified
                    IdentifierType.OrgId,
                    "18228",
                    IdentifierType.Ric)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        [Example("Entity Search: By Name")]
        public void EntitySearch_ByName()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "EntitySearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.EntitySearch(
                EntitySearchRequest.Create(
                    "Chrysler", //Name
                    false, //Offical name only
                    null, //Domicile codes
                    null, //Country of incorporation
                    null, //Industry classification
                    false, //Verified only
                    false, //Regulated only
                    false, //Issuers only
                    false, //Active only
                    false, //includeUnManagedOrUnVerified
                    IdentifierType.OrgId,
                    null,
                    IdentifierType.Ric)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region Fund Search

        [Example("Fund Search: By Fund Name")]
        public void FundSearch_ByRic()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "FundSearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.FundSearch(
                FundSearchRequest.Create(
                    null, //Asset Status
                    "Fidelity", //Fund name
                    null, //Fund adminstrator
                    null, //Portfolio manager
                    null, //Domicile code
                    null, //Currency code
                    null, //Sub types
                    IdentifierType.Ric,
                    null,
                    IdentifierType.Ric)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        [Example("Fund Search: All Options")]
        public void FundSearch_AllOptions()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, "DomicileType", "GetAll", MethodType.Operation, Publish.Primary);
            var domiciles = SearchContext.GetCountries();

            //Output
            foreach (var domcile in domiciles.Take(3))
                Status.WriteLine("{0} {1} ({2})", domcile.Code, domcile.Name, domcile.IsoCode);
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, "CurrencyType", "GetAll", MethodType.Operation, Publish.Primary);
            var currencies = SearchContext.GetCurrencies();

            //Output
            foreach (var currency in currencies.Take(3))
                Status.WriteLine("{0} {1}", currency.Code, currency.Name);
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "EntitySearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.FundSearch(
                FundSearchRequest.Create(
                    FundStatus.Active, //Asset Status
                    "Vanguard VEF", //Fund name
                    "VANGUARD INVESTMENTS CANADA INC.", //Fund adminstrator
                    "VANGUARD INVESTMENTS CANADA INC.", //Portfolio manager
                    new[] { "CA" }, //Domicile codes
                    new[] { "CAD" }, //Currency codes
                    null, /* new [] { FundSubType.ExchangeTradedFund } */
                    IdentifierType.Ric,
                    "LP68137757",
                    IdentifierType.Ric)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }
        #endregion

        #region Loan Search

        [Example("Loan Search")]
        public void LoanSearch()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "GetLoanSearchFacilityTypes", MethodType.Operation, Publish.Primary);
            var validFacilityTypes = SearchContext.GetLoanSearchFacilityTypes();
            Status.WriteLine("Loan Search Facility Types: " + String.Join(", ", validFacilityTypes.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);
            var myValidFacilityType = validFacilityTypes.FirstOrDefault().Code;  //The first one will be default or all.

            Status.Notify(SearchContext, null, "GetLoanSearchDomiciles", MethodType.Operation, Publish.Primary);
            var validDomiciles = SearchContext.GetLoanSearchDomiciles();
            Status.WriteLine("Loan Search Domiciles: " + String.Join(", ", validDomiciles.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);
            var validDomicile = validDomiciles.First(n => n.Name == "United States (US)").Code;

            Status.Notify(SearchContext, null, "GetLoanSearchBaseRates", MethodType.Operation, Publish.Primary);
            var validBaseRates = SearchContext.GetLoanSearchBaseRates();
            Status.WriteLine("Loan Search Base Rates: " + String.Join(", ", validBaseRates.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);
            var validBaseRate = validBaseRates.First(n => n.Name == "LIBOR").Code;

            Status.Notify(SearchContext, null, "GetLoanSearchCurrencies", MethodType.Operation, Publish.Primary);
            var validCurrencies = SearchContext.GetLoanSearchCurrencies();
            Status.WriteLine("Loan Search Currencies: " + String.Join(", ", validCurrencies.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);
            var validCurrency = validCurrencies.First(n => n.Name == "U.S. Dollar").Code;

            Status.Notify(SearchContext, null, "GetLoanSearchIndustries", MethodType.Operation, Publish.Primary);
            var validIndustries = SearchContext.GetLoanSearchIndustries();
            Status.WriteLine("Loan Search Industries: " + String.Join(", ", validCurrencies.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);
            var validIndustry = validIndustries.First(n => n.Name == "Manufacturing").Code;

            DateRangeComparison validDateRangeComparison = new DateRangeComparison() { From = DateTimeOffset.Now, To = DateTimeOffset.Now + TimeSpan.FromDays(180) };
            DateValueComparison validDateValueComparison = new DateValueComparison() { ComparisonOperator = ComparisonOperator.GreaterThanEquals, Value = DateTimeOffset.Now };

            NumericRangeComparison validNumericRangeComparison = new NumericRangeComparison() { From = 0, To = 1000 };
            NumericValueComparison validNumericValueComparison = new NumericValueComparison() { ComparisonOperator = ComparisonOperator.GreaterThanEquals, Value = 00.5555M };

            LoanSearchRequest lsr = LoanSearchRequest.Create(
            companyName: "Ford Motor Co",
            ticker: null,
            facilityTypeCodes: new[] { myValidFacilityType },
            domicileCodes: new[] { validDomicile },
            baseRateCodes: new[] { validBaseRate },
            currencyCodes: new[] { validCurrency },
            industryCodes: new[] { validIndustry },
            maturityDate: validDateRangeComparison,  //OR...
            //maturity: validDateValueComparison,
            margin: validNumericRangeComparison, //OR...
            //margin: validNumericValueComparison, 
            bidPrice: validNumericRangeComparison, //OR...
            //bidPrice: validNumericValueComparison,
            identifierType: IdentifierType.NONE,
            identifier: null,
            preferredIdentifierType: IdentifierType.NONE,
            activeOnly: true);

            

            Status.Notify(SearchContext, null, "LoanSearch", MethodType.Operation, Publish.Primary);
            var loanSearchResults = SearchContext.LoanSearch(lsr);


            foreach (var instrument in loanSearchResults.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region Equity Search

        [Example("Equity Search: All Options")]
        public void EquitySearch_AllOptions()
        {
            SearchContext.Preferences.MaxPageSize = 10;

			Status.Notify(SearchContext, null, "GetInstrumentSubTypes", MethodType.Operation, Publish.Primary);
			//Get the list of available instrument sub types
			var instrumentSubTypes = SearchContext.GetInstrumentSubTypes();
			Status.WriteLine("Instrument Sub Types: " +
							String.Join(", ", instrumentSubTypes.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetAssetCategories", MethodType.Operation, Publish.Primary);
			//Get the list of available asset categories
			var getAssetCategories = SearchContext.GetAssetCategories();
			Status.WriteLine("Asset Categories: " + String.Join(", ", getAssetCategories.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetEquitySubTypes", MethodType.Operation, Publish.Primary);
			//Get the list of available equity sub types
			var equitySubTypes = SearchContext.GetEquitySubTypes();
			Status.WriteLine("Equity Sub Types: " + String.Join(", ", equitySubTypes.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetCurrencies", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var currencies = SearchContext.GetCurrencies();
            Status.WriteLine("Currencies: " + String.Join(", ", currencies.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetDomiciles", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var domiciles = SearchContext.GetCountries();
            Status.WriteLine("Domiciles: " + String.Join(", ", domiciles.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetExchanges", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var exchanges = SearchContext.GetExchanges();
            Status.WriteLine("Exchanges: " + String.Join(", ", exchanges.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetGicsIndustryClassifications", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var gics = SearchContext.GetGicsIndustryClassifications();
            Status.WriteLine("Exchanges: " + String.Join(", ", gics.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "EquitySearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.EquitySearch(
                EquitySearchRequest.Create(
                    null, //Sub types
                    null, //Asset categories
                    "IBM", //Company name
                    null, //Currency codes
                    "INTERNATIONAL BUSINESS MACHINES CORPORATION", //Description
                    new [] { "US" }, //Domiciles codes
                    FairValueIndicator.Yes, //Fair value indicator
                    new [] { "NYS" }, //Exchange codes
                    null, //GICs codes
                    null, //File codes
                    "188228", //Org identifier
                    EquityStatus.Active, //Asset status
                    null, //Ticker
					IdentifierType.Ric, //Identifier types
                    "188228", //Identifier
                    IdentifierType.Ric));

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        [Example("Equity Search: By Cusip")]
        public void EquitySearch_ByCusip()
        {
            /* This search demonstrates searching for IBM by Cusip returning the RICs for all 47 exchanges
             */
            SearchContext.Preferences.MaxPageSize = 10;

			Status.Notify(SearchContext, null, "GetEquitySubTypes", MethodType.Operation, Publish.Primary);
			//Get the list of available equity sub types
			var equitySubTypes = SearchContext.GetEquitySubTypes();
			Status.WriteLine("Equity Sub Types: " + String.Join(", ", equitySubTypes.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetCurrencies", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var currencies = SearchContext.GetCurrencies();
            Status.WriteLine("Currencies: " + String.Join(", ", currencies.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetDomiciles", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var domiciles = SearchContext.GetCountries();
            Status.WriteLine("Domiciles: " + String.Join(", ", domiciles.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetExchanges", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var exchanges = SearchContext.GetExchanges();
            Status.WriteLine("Exchanges: " + String.Join(", ", exchanges.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetGicsIndustryClassifications", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var gics = SearchContext.GetGicsIndustryClassifications();
            Status.WriteLine("Exchanges: " + String.Join(", ", gics.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "EquitySearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.EquitySearch(
                EquitySearchRequest.Create(
                    null, //Sub types
                    null, //Asset categories
                    null, //Company name
                    null, //Currency codes
                    null, //Description
                    null, //Domiciles codes
                    null, //Fair value indicator
                    null, //Exchange codes
                    null, //GICs codes
                    null, //File codes
                    null, //Org identifier
                    null, //Asset status
                    null, //Ticker
					IdentifierType.Cusip, //Identifier types
                    "459200101", //Identifier
                    IdentifierType.Ric));

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region Futures and Options Search

        [Example("Futures and Options: Wildcard")]
        public void FuturesAndOptionsSearch_AllOptions()
        {
            /* Search for EuroDollar Futures that trade on the CME IMM exchange. EuroDollars are integral to every Fixed Income 
             * trading desk: RIC: ED* Exchange: CMM (IMM)
             */
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "GetFileCodes", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var availableFileCodes = SearchContext.GetFileCodes();
            Status.WriteLine("File Codes: " + String.Join(", ", availableFileCodes.Take(3).Select(fc=>fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetExchanges", MethodType.Operation, Publish.Primary);
            //Get the list of available file codes
            var availableExchanges = SearchContext.GetExchanges();
            Status.WriteLine("Exchanges: " + String.Join(", ", availableExchanges.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetFuturesAndOptionsExchangeTypes", MethodType.Operation, Publish.Primary);
            //Get the list of available get futures and options Exchange types
            var getFuturesAndOptionsExchangeTypes = SearchContext.GetFuturesAndOptionsExchangeTypes();
            Status.WriteLine("Get Futures And Options ExchangeTypes: " + String.Join(", ", getFuturesAndOptionsExchangeTypes.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);


            Status.Notify(SearchContext, null, "FuturesAndOptionsSearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.FuturesAndOptionsSearch(
                FuturesAndOptionsSearchRequest.Create(
                    null, //Futures and Options Type
                    null, //Put Call
                    null, //File Codes
                    null, //Underlying RIC
                    null, //Excersize Style
                    null, //Currency Code
                    null, //Description
                    new[] { "IMM" }, //Exchange Codes: //CME:International Monetary Market (IMM)
                    null, //Strike Price
                    null, //Expiration Date
                    null, //Status
                    IdentifierType.Ric,
                    "ED*", //Identifier: Euro Dollar
                    IdentifierType.Ric));

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region Gov Corp

	    [Example("Gov Corp: Bonds by Ticker")]
        public void GovCorpSearch_ByRic()
        {
            SearchContext.Preferences.MaxPageSize = 10;

			Status.Notify(SearchContext, null, "GetGovCorpAssetStatuses", MethodType.Operation, Publish.Primary);
			//Get the list of available GetGovCorpAssetStatuses
			var govCorpAssetStatuses = SearchContext.GetGovCorpAssetStatuses();
			Status.WriteLine("Gov Corp Asset Statuses: " +
							String.Join(", ", govCorpAssetStatuses.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetGovCorpSubGroups", MethodType.Operation, Publish.Primary);
			//Get the list of available gov corp sub groups
			var govCorpSubTypes = SearchContext.GetGovCorpSubGroups();
			Status.WriteLine("Gov Corp Sub Groups: " + String.Join(", ", govCorpSubTypes.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
			Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetGovCorpContributors", MethodType.Operation, Publish.Primary);
            //Get the list of available contributors
            var contributors = SearchContext.GetGovCorpContributors();
            Status.WriteLine("Gov Corp Contributors: " + String.Join(", ", contributors.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetCountries", MethodType.Operation, Publish.Primary);
            //Get the list of available countries
            var countries = SearchContext.GetCountries();
            Status.WriteLine("Countries: " + String.Join(", ", countries.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetCurrencies", MethodType.Operation, Publish.Primary);
            //Get the list of available currencies
            var currencies = SearchContext.GetCurrencies();
            Status.WriteLine("Currencies: " + String.Join(", ", currencies.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetIndustrySectors", MethodType.Operation, Publish.Primary);
            //Get the list of available industry sectors
            var industrySectors = SearchContext.GetIndustrySectors();
            Status.WriteLine("Industry Sectors: " + String.Join(", ", industrySectors.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetMoodyRatings", MethodType.Operation, Publish.Primary);
            //Get the list of available moody ratings
            var moodyRatings = SearchContext.GetMoodyRatings();
            Status.WriteLine("Moody Ratings: " + String.Join(", ", moodyRatings.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetStandardAndPoorsRatings", MethodType.Operation, Publish.Primary);
            //Get the list of available s&p ratings
            var standardAndPoorsRatings = SearchContext.GetStandardAndPoorsRatings();
            Status.WriteLine("Standard and Poors Ratings: " + String.Join(", ", standardAndPoorsRatings.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GovCorpSearch", MethodType.Operation, Publish.Primary);
            var results = SearchContext.GovCorpSearch(
                GovCorpSearchRequest.Create(
                    null, //Asset Status
                    null, //Gov Corp Contributor code
                    null, //Country code
                    null, //Numeric comparision
                    null, //Currency code
                    null, //Industry sector code
                    GovCorpSecurityGroups.Create(true, true, true, true, null),
                    null, //Moody's
                    null, //Standard and Poors
                    false, //Callable
                    false, //Convertable
                    false, //Extendable
                    false, //Putable
                    false, //Sinkable
                    null, //Issuer
                    null, //Issue date
                    null, //Maturity date
                    null, //Next pay date
                    null, //OrgId
                    "AAPL", //Ticker 
                    IdentifierType.Ric, //Identifier Type
                    null, //Identifier
                    IdentifierType.Ric)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region Mortgage

        [Example("Mortgage: By Agency and Coupon")]
        public void MortPassThruSearch_ByAgency()
        {
            SearchContext.Preferences.MaxPageSize = 10;

			Status.Notify(SearchContext, null, "GetMortgagePoolTypes", MethodType.Operation, Publish.Primary);
            var contributors = SearchContext.GetMortgagePoolTypes();
            Status.WriteLine("Pool Types: " + String.Join(", ", contributors.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "GetMortgageAgencies", MethodType.Operation, Publish.Secondary);
            var countries = SearchContext.GetMortgageAgencies();
            Status.WriteLine("Agencies: " + String.Join(", ", countries.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "MortgageSearch", MethodType.Operation, Publish.Primary);
            //Searching for high coupon 10 percent MBS from Fannie Mae
            var results = SearchContext.MortgageSearch(
                MortgageSearchRequest.Create(
                    "0x000019000050662f", //Agency Code: FNMA
                    null, //Pool Number
                    null, //Pool Type
                    NumericComparison.CreateNumericValueComparison(ComparisonOperator.GreaterThanEquals, 10), //Coupon (numeric comparison)
                    null, //Amortization type
                    null, //Status
                    null, //Security group
                    null, //Settle month
                    IdentifierType.Cusip, //Identifier Type
                    null, //Identifier
                    IdentifierType.Cusip)); //Preferred Identifier Type

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region US Municipals

        [Example("Us Municipals: By Coupon")]
        public void UsMunicipals_ByCoupon()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "GetMoodyRatings", MethodType.Operation, Publish.Primary);
            var moodyRatings = SearchContext.GetMoodyRatings();
            Status.WriteLine("Moody Ratings: " + String.Join(", ", moodyRatings.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetStandardAndPoorsRatings", MethodType.Operation, Publish.Secondary);
            var standardAndPoorsRatings = SearchContext.GetStandardAndPoorsRatings();
            Status.WriteLine("S & P Ratings: " + String.Join(", ", standardAndPoorsRatings.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetStates", MethodType.Operation, Publish.Secondary);
            var states = SearchContext.GetStates();
            Status.WriteLine("States: " + String.Join(", ", states.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "UsMunicipalSearch", MethodType.Operation, Publish.Primary);
            //Search for all MUNIs from New York State with a coupon greater than 9 percent for those income looking investors
            var results = SearchContext.UsMunicipalSearch(
                UsMunicipalSearchRequest.Create(
                    null, //Status
                    NumericComparison.CreateNumericValueComparison(ComparisonOperator.GreaterThanEquals, 10), //Coupon
                    true, //Callable
                    true, //Putable
                    true, //Sinkable
                    null, //Issue description
                    null, //Maturity date
                    null, //Moody's
                    null, //Standard and Poors
                    "NY", //State
                    IdentifierType.Cusip,
                    null,
                    IdentifierType.Cusip)); //Preferred identifier

            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region CMO / ABS

        [Example("CMO ABS: By Security Group")]
        public void CmoAbs_Any()
        {
            SearchContext.Preferences.MaxPageSize = 10;

            Status.Notify(SearchContext, null, "GetCmoAbsSubGroupTypes", MethodType.Operation, Publish.Secondary);
            var standardAndPoorsRatings = SearchContext.GetCmoAbsSubGroupTypes();
            Status.WriteLine("S & P Ratings: " + String.Join(", ", standardAndPoorsRatings.Take(3).Select(fc => fc.Name + " (" + fc.Code + ")")));
            Status.EndNotify(SearchContext);

			Status.Notify(SearchContext, null, "CmoAbsSearch", MethodType.Operation, Publish.Primary);
            //CMO: Search for all Equipment backed loans
            var results = SearchContext.CmoAbsSearch(
                CmoAbsSearchRequest.Create(
                    CmoAbsSecurityGroup.Create(true, true, true, true, true, "EQIP"), //Security Group: Equipment Backed Loaned
                    null, //Issue
                    null, //Tranche
                    null, //Series
                    null, //Coupon rate
                    null, //Asset statuses
                    null, //currency code
                    IdentifierType.Cusip,
                    null,
                    IdentifierType.Cusip)); //Preferred identifier
            //Output
            foreach (var instrument in results.Take(10))
                Status.WriteLine("{0}:{1} {2} ({3})", instrument.Identifier, instrument.IdentifierType, instrument.Description, instrument.InstrumentType);
            Status.EndNotify(SearchContext);
        }

        #endregion

        #region Commodities

        [Example("Commodities Search: By RIC")]
        public void CommoditiesSearch_ByRic()
        {
            SearchContext.Preferences.MaxPageSize = 1000;
            Status.Notify(SearchContext, null, "CommoditySearch", MethodType.Operation, Publish.Primary);

            var request = new CommoditySearchRequest()
            {
                IdentifierType = IdentifierType.Ric,
                Identifier = "XAU-XAG",
                PreferredIdentifierType = IdentifierType.Ric,
            };

            var commodities = SearchContext.CommoditySearch(request);

            foreach (var commodity in commodities)
                Status.WriteLine("{0}:{1} Description=\"{2}\" Exchange={3}", commodity.Identifier, commodity.IdentifierType, commodity.Description, commodity.ExchangeCode);
            
            Status.EndNotify(SearchContext);
        }

        [Example("Commodities Search: AND 2 Fields")]
        public void CommoditiesSearch_ByDescriptionAndCurrency()
        {
            SearchContext.Preferences.MaxPageSize = 1000;
            Status.Notify(SearchContext, null, "CommoditySearch", MethodType.Operation, Publish.Primary);

            var request = new CommoditySearchRequest()
            {
                Description = "Gold",
                CurrencyCodes = new [] { "USD" },
            };

            var commodities = SearchContext.CommoditySearch(request);

            foreach (var commodity in commodities)
                Status.WriteLine("{0}:{1} Description=\"{2}\" Exchange={3} Currency={4}", commodity.Identifier, commodity.IdentifierType, commodity.Description, commodity.ExchangeCode, commodity.CurrencyCode);

            Status.EndNotify(SearchContext);
        }

        [Example("Commodities Search: OR 2 Values")]
        public void CommoditiesSearch_AnyTwoExchanges()
        {
            SearchContext.Preferences.MaxPageSize = 1000;
            Status.Notify(SearchContext, null, "CommoditySearch", MethodType.Operation, Publish.Primary);

            var request = new CommoditySearchRequest()
            {
                ExchangeCodes = new[] { "BUD", "CME" },
            };

            var commodities = SearchContext.CommoditySearch(request);

            foreach (var commodity in commodities)
                Status.WriteLine("{0}:{1} Description=\"{2}\" ExchangeCode={3} ExchangeCodeDescription=\"{4}\"", commodity.Identifier, commodity.IdentifierType, commodity.Description, commodity.ExchangeCode, commodity.ExchangeCodeDescription);

            Status.EndNotify(SearchContext);
        }

        #endregion

        #region MiFID Sub-Class

        [Example("MiFID Sub-Class Search: Keyword Search")]
        public void MifidSubclassSearch_KeywordSearch()
        {
            SearchContext.Preferences.MaxPageSize = 1000;
            Status.Notify(SearchContext, null, "MifidSubclassSearch", MethodType.Operation, Publish.Primary);

            var request = new MifidSubclassSearchRequest()
            {
                KeywordQuery = "209469330760",
            };

            var subclasses = SearchContext.MifidSubclassSearch(request);

            foreach (var subclass in subclasses)
                Status.WriteLine("{0}:{1} Sub-Asset Class Code=\"{2}\" Underlying Commodity={3}", subclass.Identifier, subclass.IdentifierType, subclass.SubAssetClassCode, subclass.UnderlyingCommodity);

            Status.EndNotify(SearchContext);
        }

        [Example("MiFID Sub-Class Search: By Perm ID")]
        public void MifidSubclassSearch_ByPermId()
        {
            SearchContext.Preferences.MaxPageSize = 1000;
            Status.Notify(SearchContext, null, "MifidSubclassSearch", MethodType.Operation, Publish.Primary);

            var request = new MifidSubclassSearchRequest()
            {
                IdentifierType = IdentifierType.Pid,
                Identifier = "209198760331",
                PreferredIdentifierType = IdentifierType.Pid,
            };

            var subclasses = SearchContext.MifidSubclassSearch(request);

            foreach (var subclass in subclasses)
                Status.WriteLine("{0}:{1} Sub-Asset Class Code=\"{2}\" Underlying Commodity={3}", subclass.Identifier, subclass.IdentifierType, subclass.SubAssetClassCode, subclass.UnderlyingCommodity);

            Status.EndNotify(SearchContext);
        }

        #endregion

        #region OTCS

        [Example("OTC Derivatives Search: By ISIN")]
        public void OtcDerivativesSearch_ByIsin()
        {
            SearchContext.Preferences.MaxPageSize = 250;
            Status.Notify(SearchContext, null, "OtcsSearch", MethodType.Operation, Publish.Primary);

            var request = new OtcsSearchRequest()
            {
                IdentifierType = IdentifierType.Isin,
                Identifier = "EZ1DZS7XQMZ9",
                AssetStatus = OtcsStatus.Active
            };

            var otcDerivatives = SearchContext.OtcsSearch(request);

            foreach (var otcDerivative in otcDerivatives)
                Status.WriteLine("{0}:{1} Description=\"{2}\" Expiration Date={3}", otcDerivative.Identifier, otcDerivative.IdentifierType, otcDerivative.Description, otcDerivative.ExpirationDate);

            Status.EndNotify(SearchContext);
        }

		#endregion

		#region Benchmark Search

		[Example("Benchmark Search: By Instrument Identifier")]
	    public void BenchmarkSearch_ByInstumentIdentifier()
		{
			SearchContext.Preferences.MaxPageSize = 5; // Setter for desired Instruments count.

			Status.Notify(SearchContext, null, "BenchmarkSearch", MethodType.Operation, Publish.Primary);

			// Benchmark Search : Search for all Instrument Identifer that starts with A.
			var validatedInstruments = SearchContext.BenchmarkSearch(
				BenchmarkSearchRequest.Create(
					description: null,
					identifierType: IdentifierType.Ric,  // Should be IdentifierType.Ric 
					identifier: "A*",
					preferredIdentifierType: IdentifierType.Ric // Should be IdentifierType.Ric | IdentifierType.NONE
				)
			); 

			//Output
			foreach (var vi in validatedInstruments)
			{
				Status.WriteLine("BenchmarkSearchResult > {0} : {1}, {2} : {3}, {4} : {5}, {6} : {7}, {8} : {9}, {10} : {11} ",
					"Identifier", vi.Identifier, 
					"Key", vi.Key, 
					"Description", vi.Description, 
					"IdentifierType", vi.IdentifierType.ToString(),
					"InstrumentType", vi.InstrumentType.ToString(),
					"Status", vi.Status.ToString());
				Status.WriteLine("");
			}

			Status.EndNotify(SearchContext);
		}

	    #endregion

	}
}
